// TODO(jan): Add nibbana-types to dependencies
// TODO(jan): Think of some way to log the place in the code the log occurred in
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Lock } from 'semaphore-async-await';
import axios from 'axios';
import freshId from 'fresh-id';
const merge = require('lodash.merge');
import * as asyncStorageUtils from 'react-native-async-storage-utils';
export const ASYNC_STORAGE_KEY = 'com.primlo.nibbana.logEntries';
const NO_CONFIG_ERROR_MESSAGE = '[nibbana] You must call nibbana.configure before any other methods';
const { NODE_ENV } = global.process.env;
// Since there's no way to atomically execute array operations on AsyncStorage,
// we need to synchronize our access to avoid race conditions.
const entriesLock = new Lock();
// This is to avoid having multiple uploads run in parallel which would cause
// entries to be logged multiple times
const uploadLock = new Lock();
let config = null;
// These are added to each entry
let superProperties = {};
let automaticUploadsIntervalId = null;
export const configure = (props) => {
    let capacity;
    if (props.capacity == null) {
        capacity = 0;
    }
    else {
        capacity = props.capacity;
    }
    let additionalHTTPHeaders = props.additionalHTTPHeaders || (() => ({}));
    let asyncStorage;
    if (props.asyncStorage) {
        asyncStorage = props.asyncStorage;
    }
    else {
        const { AsyncStorage } = require('react-native');
        asyncStorage = AsyncStorage;
    }
    let outputToConsole;
    if (props.outputToConsole == null) {
        outputToConsole = NODE_ENV !== 'production';
    }
    else {
        outputToConsole = props.outputToConsole;
    }
    const uploadEntries = (entries) => {
        const headers = additionalHTTPHeaders();
        return axios({
            method: 'POST',
            url: props.endpoint,
            data: { nibbanaToken: props.nibbanaToken, entries },
            headers,
            timeout: 15 * 1000,
        });
    };
    config = {
        uploadEntries,
        asyncStorage,
        outputToConsole,
        capacity,
    };
};
export const configureWithCustomUploadFunction = (props) => {
    let capacity;
    if (props.capacity == null) {
        capacity = 0;
    }
    else {
        capacity = props.capacity;
    }
    let asyncStorage;
    if (props.asyncStorage) {
        asyncStorage = props.asyncStorage;
    }
    else {
        const { AsyncStorage } = require('react-native');
        asyncStorage = AsyncStorage;
    }
    let outputToConsole;
    if (props.outputToConsole == null) {
        outputToConsole = NODE_ENV !== 'production';
    }
    else {
        outputToConsole = props.outputToConsole;
    }
    config = {
        uploadEntries: props.uploadEntries,
        asyncStorage,
        outputToConsole,
        capacity,
    };
};
/**
 * Sets your super properties. Super properties get saved with every entry.
 * @param newSuperProperties The new super properties
 */
export const setSuperProperties = (newSuperProperties) => {
    clearSuperProperties();
    extendSuperProperties(newSuperProperties);
};
/**
 * Extends your existing super properties
 * @param additionalSuperProperties An object to be merged into your super properties
 */
export const extendSuperProperties = (additionalSuperProperties) => merge(superProperties, additionalSuperProperties);
/**
 * Resets all super properties
 */
export const clearSuperProperties = () => {
    superProperties = {};
};
const appendEntry = (severity, data) => __awaiter(this, void 0, void 0, function* () {
    if (config === null) {
        throw new Error(NO_CONFIG_ERROR_MESSAGE);
    }
    const entryData = data.map(datum => {
        if (datum instanceof Error) {
            return {
                message: datum.message,
                name: datum.name,
                stack: datum.stack,
            };
        }
        return datum;
    });
    const newEntry = {
        _id: freshId(15),
        severity,
        occurredAt: new Date(),
        superProperties,
        data: entryData,
    };
    yield entriesLock.acquire();
    try {
        yield asyncStorageUtils.enqueueWithCapacity(config.asyncStorage, ASYNC_STORAGE_KEY, newEntry, config.capacity);
    }
    finally {
        entriesLock.release();
    }
});
/**
 * Clears all locally saved log entries.
 */
export const clearEntries = () => __awaiter(this, void 0, void 0, function* () {
    if (config === null) {
        throw new Error(NO_CONFIG_ERROR_MESSAGE);
    }
    yield entriesLock.acquire();
    try {
        yield config.asyncStorage.removeItem(ASYNC_STORAGE_KEY);
    }
    finally {
        entriesLock.release();
    }
});
const consoleProps = {
    log: console.log,
    warn: console.warn,
    debug: console.debug,
    error: console.error,
};
const newEntry = (severity, data) => __awaiter(this, void 0, void 0, function* () {
    if (config === null) {
        throw new Error(NO_CONFIG_ERROR_MESSAGE);
    }
    if (config.outputToConsole) {
        consoleProps[severity](...data);
    }
    return appendEntry(severity, data);
});
/**
 * Use this instead of console.log.
 * @param data What to log. Can be a string or an Error.
 */
export const log = (...data) => __awaiter(this, void 0, void 0, function* () { return newEntry('log', data); });
/**
 * Use this instead of console.warn.
 * @param data What to log. Can be a string or an Error.
 */
export const warn = (...data) => __awaiter(this, void 0, void 0, function* () { return newEntry('warn', data); });
/**
 * Use this instead of console.debug.
 * @param data What to log. Can be a string or an Error.
 */
export const debug = (...data) => __awaiter(this, void 0, void 0, function* () { return newEntry('debug', data); });
/**
 * Use this instead of console.error.
 * @param data What to log. Can be a string or an Error.
 */
export const error = (...data) => __awaiter(this, void 0, void 0, function* () { return newEntry('error', data); });
/**
 * This forces an upload of all locally saved log entries
 */
export const uploadNow = () => __awaiter(this, void 0, void 0, function* () {
    if (config === null) {
        throw new Error(NO_CONFIG_ERROR_MESSAGE);
    }
    yield uploadLock.acquire();
    try {
        const entriesToBeUploaded = yield asyncStorageUtils.getArray(config.asyncStorage, ASYNC_STORAGE_KEY);
        if (entriesToBeUploaded.length === 0) {
            return;
        }
        yield config.uploadEntries(entriesToBeUploaded);
        const idsOfUploadedEntries = entriesToBeUploaded.map((e) => e._id);
        yield entriesLock.acquire();
        try {
            yield asyncStorageUtils.filterArray(config.asyncStorage, ASYNC_STORAGE_KEY, e => !idsOfUploadedEntries.includes(e._id));
        }
        finally {
            entriesLock.release();
        }
    }
    finally {
        uploadLock.release();
    }
});
/**
 * This starts periodic uploads of locally saved entries in the background
 * @param frequencyInSeconds The upload frequency in seconds. Don't set this to a value too low.
 */
export const startAutomaticUploads = (frequencyInSeconds = 5 * 60) => {
    if (automaticUploadsIntervalId !== null) {
        console.log(`[nibbana] automatic uploads already in progress`);
        return;
    }
    automaticUploadsIntervalId = setInterval(uploadNow, frequencyInSeconds * 1000);
};
/**
 * This stops periodic background uploads
 */
export const stopAutomaticUploads = () => {
    if (automaticUploadsIntervalId !== null) {
        clearInterval(automaticUploadsIntervalId);
    }
};
